# -*- coding: utf-8 -*-
# Copyright (C) 2018 Freetech Solutions

from django.db import models
from django.db.models import fields
from django.utils.translation import gettext_lazy as _

from ominicontacto_app.models import Campana, ArchivoDeAudio, Contacto, AgenteProfile
from configuracion_telefonia_app.models import DestinoEntrante


class Encuesta(models.Model):

    ESTADO_INACTIVA = 0
    ESTADO_ACTIVA = 1
    ESTADO_ELIMINADA = 2
    ESTADOS = (
        (ESTADO_INACTIVA, _('Inactiva')),
        (ESTADO_ACTIVA, _('Activa')),
        (ESTADO_ELIMINADA, _('Eliminada')),
    )

    nombre = models.CharField(max_length=100, verbose_name=_('Nombre'))
    time_out = models.PositiveIntegerField()
    time_retry = models.PositiveIntegerField()
    max_retries = models.PositiveIntegerField()
    audio_inicio = models.ForeignKey(ArchivoDeAudio, blank=True, null=True,
                                     on_delete=models.SET_NULL,
                                     related_name='survey_audio_inicio')
    audio_fin = models.ForeignKey(ArchivoDeAudio, blank=True, null=True,
                                  on_delete=models.SET_NULL,
                                  related_name='survey_audio_fin')

    estado = models.PositiveIntegerField(
        choices=ESTADOS,
        default=ESTADO_INACTIVA,
    )

    # Codigos de error al verificar si se puede activar
    PREGUNTA_SIN_OPCIONES = 1
    PREGUNTA_NO_ALCANZABLE = 2
    CONTIENE_CICLOS = 3
    SIN_PREGUNTA_INICIAL = 4
    MULTIPLE_PREGUNTA_INICIAL = 5


class PreguntaDeEncuesta(models.Model):
    encuesta = models.ForeignKey(Encuesta, on_delete=models.CASCADE, related_name='preguntas')
    nombre = models.CharField(max_length=100, verbose_name=_('Nombre'))
    descripcion = models.TextField(verbose_name=_('Descripción'))
    es_ponderable = models.BooleanField()
    audio = models.ForeignKey(ArchivoDeAudio, blank=True, null=True,
                              on_delete=models.SET_NULL,
                              related_name='survey_audio_pregunta')
    es_pregunta_inicial = models.BooleanField(default=False)


class OpcionDePreguntaDeEncuesta(models.Model):
    ACCION_FIN = 0
    ACCION_PREGUNTA = 1
    ACCION_DESTINO_ENTRANTE = 2

    ACCIONES = (
        (ACCION_FIN, _('Finalizar')),
        (ACCION_PREGUNTA, _('Siguiente Pregunta')),
        (ACCION_DESTINO_ENTRANTE, _('Destino Entrante')),
    )

    OPCIONES_DTMF = tuple([(str(i), str(i)) for i in range(100)])

    pregunta = models.ForeignKey(
        PreguntaDeEncuesta, on_delete=models.CASCADE, related_name='opciones')
    dtmf = models.CharField(choices=OPCIONES_DTMF, max_length=2, verbose_name=_('DTMF'))
    calificacion = models.CharField(max_length=100, verbose_name=_('Calificación'))
    accion = models.PositiveIntegerField(
        choices=ACCIONES,
        default=ACCION_FIN,
    )
    pregunta_siguiente = models.ForeignKey(
        PreguntaDeEncuesta, on_delete=models.CASCADE, related_name='opciones_anteriores',
        blank=True, null=True)
    destino = models.ForeignKey(DestinoEntrante, related_name='opciones_de_encuesta',
                                blank=True, null=True, on_delete=models.PROTECT)

    class Meta:
        unique_together = ('pregunta', 'dtmf')


class EncuestaEnCampana(models.Model):
    campana = models.ForeignKey(
        Campana, on_delete=models.CASCADE, related_name='encuestas')
    encuesta = models.ForeignKey(
        Encuesta, on_delete=models.CASCADE, related_name='campanas')
    activa = fields.BooleanField(default=True)

    class Meta:
        unique_together = ('campana', 'encuesta')


class RespuestaDePreguntaDeEncuesta(models.Model):
    pregunta = models.ForeignKey(
        PreguntaDeEncuesta, on_delete=models.CASCADE, related_name='respuestas')
    opcion = models.ForeignKey(
        OpcionDePreguntaDeEncuesta, on_delete=models.CASCADE, related_name='respuestas',
        blank=True, null=True)
    campana = models.ForeignKey(
        Campana, on_delete=models.CASCADE, related_name='respuestas_de_encuesta')
    callid = models.CharField(max_length=32, blank=True, null=True)
    callerid = models.CharField(max_length=100)
    contacto = models.ForeignKey(
        Contacto, on_delete=models.CASCADE, related_name='respuestas', blank=True, null=True)
    agente = models.ForeignKey(
        AgenteProfile, on_delete=models.CASCADE, related_name='respuestas',
        blank=True, null=True)
    grabacion = models.CharField(max_length=100, blank=True, null=True)
    fecha = models.DateTimeField()
